#include "tbs.h"

BOOL exit_flag = FALSE;

void term_signal(int signum) {
    exit_flag = TRUE;
}

void int_signal(int signum) {
    exit_flag = TRUE;
}


void mac_addr_n2a(char *mac_addr, unsigned char *arg)
{
	int i, l;

	l = 0;
	for (i = 0; i < ETH_ADDR_LEN ; i++) {
		if (i == 0) {
			sprintf(mac_addr+l, "%02x", arg[i]);
			l += 2;
		} else {
			sprintf(mac_addr+l, ":%02x", arg[i]);
			l += 3;
		}
	}
}


#ifdef HAVE_NETLINK_MSG_H
#if !defined(CONFIG_LIBNL20) && !defined(CONFIG_LIBNL30)
/* libnl 2.0 compatibility code */

static inline struct nl_handle *nl_socket_alloc(void)
{
	return nl_handle_alloc();
}

static inline void nl_socket_free(struct nl_handle *h)
{
	nl_handle_destroy(h);
}

static inline int __genl_ctrl_alloc_cache(struct nl_handle *h, struct nl_cache **cache)
{
	struct nl_cache *tmp = genl_ctrl_alloc_cache(h);
	if (!tmp)
		return -ENOMEM;
	*cache = tmp;
	return 0;
}
#define genl_ctrl_alloc_cache __genl_ctrl_alloc_cache
#endif /* CONFIG_LIBNL20 */

static int nl80211_init(struct nl80211_info *info) {
	int errcode;

	info->state.nl_sock = nl_socket_alloc();
	if (!info->state.nl_sock) {
		log_debug("Failed to allocate netlink socket.");
		return -ENOMEM;
	}

	if (genl_connect(info->state.nl_sock)) {
		log_debug("Failed to connect to generic netlink.");
		errcode = -ENOLINK;
		goto out_handle_destroy;
	}

	if (genl_ctrl_alloc_cache(info->state.nl_sock, &info->state.nl_cache)) {
		log_debug("Failed to allocate generic netlink cache.");
		errcode = -ENOMEM;
		goto out_handle_destroy;
	}

	info->state.nl80211 = genl_ctrl_search_by_name(info->state.nl_cache, "nl80211");
	if (!info->state.nl80211) {
		log_debug("nl80211 not found.");
		errcode = -ENOENT;
		goto out_cache_free;
	}

	return 0;

 out_cache_free:
	nl_cache_free(info->state.nl_cache);
 out_handle_destroy:
	nl_socket_free(info->state.nl_sock);
	return errcode;
}

int nl80211_send_simple_cmd( struct nl80211_info * info, int cmd, int cmd_flag, nl80211_data_prep f, void * callback) {
    BOOL	res = FALSE;
    struct nl_msg *	msg;

    if ( info == NULL) {
	log_debug("NULLified ptr was sent to NL80211");
	return res;
    }
    if ( nl80211_init( info) != 0) {
	log_debug("Could not initialize NL80211 Layer!");
	return res;
    }
    if ((msg = nlmsg_alloc()) != NULL) {
	genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, genl_family_get_id(info->state.nl80211), 0, cmd_flag, cmd, 0);
	if ( f != NULL)
	    (*f)(info, msg);
	if (nl_send_auto_complete(info->state.nl_sock, msg) < 0) {
	    log_debug("Something goes bad!");
	    goto out;
	}
	nl_socket_modify_cb(info->state.nl_sock, NL_CB_VALID, NL_CB_CUSTOM, callback, info);
	nl_recvmsgs_default(info->state.nl_sock);
	res = TRUE;
out:
	nlmsg_free(msg);
    }
    genl_family_put(info->state.nl80211);
    nl_cache_free(info->state.nl_cache);
    nl_socket_free(info->state.nl_sock);
    return res;
}

void iface_prepare_data(struct nl80211_info * info, struct nl_msg * msg) {
    if ((info != NULL) && (msg != NULL))
	switch (info->idby) {
	case CIB_PHY:
		nla_put_u32(msg, NL80211_ATTR_WIPHY, info->phyidx);
		break;
	case CIB_NETDEV:
		nla_put_u32(msg, NL80211_ATTR_IFINDEX, info->devidx);
		break;
	default:
		break;
	}
}

#endif
