#define PROGNAME "tbs-client"
#include "tbs.h"
#include <net/if.h>

static unsigned int ifindex;

static void usage( void) {
    fprintf(stderr, "Usage: tbs_client IFNAMRE NUMBER [[NUMBER]]\n");
    exit(1);
}

static void sync_prepare_data(struct nl80211_info * info, struct nl_msg * msg) {
    if ((info != NULL) && (msg != NULL)) {
	struct tdma_tsf * tsf = info->localcfg;
	nla_put(msg, NL80211_ATTR_TDMA_EXT_TSF, sizeof(struct tdma_tsf), tsf);
	nla_put_u32(msg, NL80211_ATTR_IFINDEX, info->devidx);
    }
}


static void my_read( struct listen_addr * la) {
    struct tdma_tsf tsf;
    ssize_t sz = 0;
    struct nl80211_info	info;

    memset(&tsf, 0, sizeof(tsf));
    sz = sock_read( la->fd, &tsf, sizeof(tsf), 10);
    if ( sz == sizeof(tsf)) {
	memset( &info, 0, sizeof(struct nl80211_info));
	info.idby = CIB_NETDEV;
	info.devidx = ifindex;
	nl80211_send_simple_cmd( &info, NL80211_CMD_EXT_SYNC_TDMA, 0, sync_prepare_data, NULL);
    }
}

int main( int argc, char * argv[]) {
    int rc;
    struct listen_addr * la;
    struct join_group * j;
    struct join_groups * join;
    struct listen_addrs * addrs;
    POLL_T * p;
    size_t i;
    char * ptr;
    struct sockaddr_in maddr;


    plib_init();
    log_open(PROGNAME, L_INFO|L_NOTICE|L_DBG|L_ERR);
    if ( argc < 3)
	usage();
    if ((ifindex = if_nametoindex(argv[1])) == 0) {
	log_err("if_nametoindex: could not convert");
    }
    i = 2;
    if ((addrs = (struct listen_addrs *)ZALLOC(sizeof(struct listen_addrs))) == NULL) {
	log_err("Could not allocate memory for listen addrs\n");
	exit(1);
    }
    addrs->tqh_last = &addrs->tqh_first;
    while ( i < argc) {
	if ((join = (struct join_groups *)ZALLOC(sizeof(struct join_groups))) == NULL) {
	    log_err("Could not allocate memory for join groups\n");
	    exit(1);
	}
	join->tqh_last = &join->tqh_first;
	if ((j = (struct join_group *)ZALLOC(sizeof(struct join_group))) == NULL) {
	    log_err("Could not allocate memory for join group\n");
	    exit(1);
	}
	memset((void *)&maddr, 0, sizeof(maddr));
	maddr.sin_addr.s_addr = htonl(BASE_ADDRESS+atoi(argv[i]));
	maddr.sin_port = htons(BASE_PORT+((unsigned short)atoi(argv[i])*10));
	maddr.sin_family = AF_INET;
	addr_ston((const struct sockaddr *)&maddr, &j->addr);
	TAILQ_INSERT_TAIL( join, j, entry);
	if ((la = (struct listen_addr *)ZALLOC(sizeof(struct listen_addr))) == NULL) {
	    log_err("Could not allocate memory for listen addr\n");
	    exit(1);
	}
	la->option = SOCK_TYPE_UDP|SOCK_OPT_RECV_MIN|SOCK_OPT_SEND_MIN;
	addr_pton("0.0.0.0", &la->addr);
	la->port = (unsigned short)BASE_PORT+(unsigned short)atoi(argv[i]);
	la->f = my_read;
	la->join = join;
	TAILQ_INSERT_TAIL( addrs, la, entry);
	i ++;
    }
    if ((p = sock_init_read( addrs)) == NULL)
	log_err("Could not allocate memory!");
    port_signal(SIGINT, int_signal);
    port_signal(SIGTERM, term_signal);
    while (!exit_flag) {
	sock_loop_read(p);
    }
out:
    sock_close_read( p);
    TAILQ_FOREACH(la, addrs, entry) {
	TAILQ_FOREACH( j, la->join, entry) {
	    TAILQ_REMOVE(la->join, j, entry);
	    ZFREE( j);
	}
	ZFREE( la->join);
	TAILQ_REMOVE(addrs, la, entry);
	ZFREE( la);
    }
    ZFREE(addrs);
    plib_close();
    return 0;
}
