#define PROGNAME "tbs-server"
#include "tbs.h"
#include <net/if.h>

static int msock_open( void) {
    int	fd;
    int 	i;
    struct in_addr iface;

    if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
	log_err("sock_open: socket");
	return (-1);
    }
    iface.s_addr = htonl(INADDR_ANY);
    if ( setsockopt( fd, IPPROTO_IP, IP_MULTICAST_IF, (char *)&iface, sizeof(iface)) < 0) {
	log_err("sock_open: could not setup multicast interface");
	close( fd);
	return (-1);
    }
    unsigned char ttl = (unsigned char)255;
    setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
    i = 1024;
    log_debug("Setting socket recieve buf to %d", i);
    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &i, sizeof(i)) < 0)
	log_debug("setsockopt(SO_RCVBUF)");
    return fd;
}

static void usage( void) {
    fprintf(stderr, "Usage: tbs-server IFNAME NUMBER\n");
    exit(1);
}

#ifdef HAVE_NETLINK_MSG_H
static int get_sync_handler(struct nl_msg *msg, void *arg)
{
	struct genlmsghdr *gnlh = nlmsg_data(nlmsg_hdr(msg));
	struct nlattr *tb_msg[NL80211_ATTR_MAX + 1];
	struct nl80211_info	* info = arg;
	struct tdma_tsf	* tsf;
	
	if (info != NULL)
	    tsf = (struct tdma_tsf *)info->localcfg;

	nla_parse(tb_msg, NL80211_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
		  genlmsg_attrlen(gnlh, 0), NULL);

	if (tb_msg[NL80211_ATTR_TDMA_EXT_TSF]) {
		if ( tsf != NULL)
		    memcpy(tsf, nla_data(tb_msg[NL80211_ATTR_TDMA_EXT_TSF]), sizeof(struct tdma_tsf));
	}
	return NL_SKIP;
}
#endif

BOOL	get_sync( unsigned int iface, struct tdma_tsf * tsf) {
#ifdef HAVE_NETLINK_MSG_H
    struct nl80211_info	info;

    if ((tsf == NULL) || (iface == 0))
	return FALSE;
    memset( &info, 0, sizeof(struct nl80211_info));
    memset( tsf, 0, sizeof(struct tdma_tsf));
    info.idby = CIB_NETDEV;
    info.devidx = iface;
    info.localcfg = tsf;
    nl80211_send_simple_cmd( &info, NL80211_CMD_QUERY_SYNC_TDMA, 0, iface_prepare_data, get_sync_handler);
    return TRUE;
#else
    return FALSE;
#endif
}

int main( int argc, char * argv[]) {
    int rc, fd;
    struct tdma_tsf tsf;
    struct sockaddr_in maddr;
    size_t size;
    unsigned int ifindex;


    plib_init();
    log_open(PROGNAME, L_INFO|L_NOTICE|L_DBG|L_ERR);
    if ( argc < 3)
	usage();
    if ((ifindex = if_nametoindex(argv[1])) == 0) {
	log_err("if_nametoindex: could not convert");
    }
    if ((fd = msock_open()) < 0) {
	log_err("sock_open: socket");
    }
    memset((void *)&maddr, 0, sizeof(maddr));
    maddr.sin_addr.s_addr = htonl(BASE_ADDRESS+atoi(argv[2]));
    maddr.sin_port = htons(BASE_PORT+((unsigned short)atoi(argv[2])*10));
    maddr.sin_family = AF_INET;

    port_signal(SIGINT, int_signal);
    port_signal(SIGTERM, term_signal);

    while (!exit_flag) {
	if ( get_sync(ifindex, &tsf)) {
	    rc = sendto(fd, &tsf, sizeof(tsf), 0, (void *)&maddr, sizeof(maddr));
	    if (rc != size)
		log_debug("short write: wrote %d bytes. Errno is %d. %s\n", rc, errno, strerror(errno));
	}
	sleep(10);
    }
    close(fd);
    plib_close();
    return 0;
}
